import sdl2
import sdl2.ext

import threading

from aerforgesdl.color import Color

class GameObject:
    def __init__(self, window, width = 20, height = 20, x = 0, y = 0, color = Color(240, 240, 240)):
        self.entity = sdl2.ext.Entity(world = window.world)

        self.window = window

        self.width = width
        self.height = height

        self.x = x
        self.y = y

        self.color = color

        self._destroyed = False
        self._visible = True

        self.entity.sprite = self.entity.world.factory.from_color(self.color, (self.width, self.height))
        self.entity.sprite.position = self.x, self.y

    def _update(self):
        self.update()

    def update(self):
        if not self._destroyed:
            if self._visible:
                self.entity.sprite = self.entity.world.factory.from_color(self.color, (self.width, self.height))

            else:
                self.entity.sprite = self.entity.world.factory.from_color(self.color, (0, 0))

            self.entity.sprite.position = self.x, self.y

        else:
            self.entity.sprite = self.entity.world.factory.from_color(self.color, (0, 0))
            self.entity.sprite.position = 0, 0

    def set_visible(self, visible):
        if not self._destroyed:
            if not visible:
                self.entity.sprite = self.entity.world.factory.from_color(self.color, (0, 0))
                self._visible = False

            else:
                self.entity.sprite = self.entity.world.factory.from_color(self.color, (self.width, self.height))
            
    def hit(self, gameobject):
        if not self._destroyed:
            left, top, right, bottom = self.entity.sprite.area
            bleft, btop, bright, bbottom = gameobject.entity.sprite.area

            return (bleft < right and bright > left and btop < bottom and bbottom > top)

        else:
            return False

    def destroy(self):
        self.entity.sprite = self.entity.world.factory.from_color(self.color, (0, 0))
        self._destroyed = True
    
    def center(self):
        if not self._destroyed:
            self.x = int(self.window.width / 2) - int(self.width / 2)
            self.y = int(self.window.height / 2) - int(self.height / 2)

    def center_x(self):
        if not self._destroyed:
            self.x = int(self.window.width / 2) - int(self.width / 2)

    def center_y(self):
        if not self._destroyed:
            self.y = int(self.window.height / 2) - int(self.height / 2)